/*# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image
#                        pour la Sant�)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
# Previous Authors : Laurent Guigues, Jean-Pierre Roux
# CreaTools website : www.creatis.insa-lyon.fr/site/fr/creatools_accueil
#
#  This software is governed by the CeCILL-B license under French law and
#  abiding by the rules of distribution of free software. You can  use,
#  modify and/ or redistribute the software under the terms of the CeCILL-B
#  license as circulated by CEA, CNRS and INRIA at the following URL
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability.
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------ */

#include "bbmaracasvisuAxeVolume.h"
#include "bbcreaMaracasVisuPackage.h"
namespace bbcreaMaracasVisu
{

BBTK_ADD_BLACK_BOX_TO_PACKAGE(creaMaracasVisu,AxeVolume)
BBTK_BLACK_BOX_IMPLEMENTATION(AxeVolume,bbtk::AtomicBlackBox);
void AxeVolume::Process()
{

	printf("EED AxeVolume::Process start \n");

	if ( mimage!=NULL )
	{
		mimage->Delete();
	}

	int ext[6];
	bbGetInputIn()->GetExtent(ext);
	int sizeX=ext[1]-ext[0]+1;
	int sizeY=ext[3]-ext[2]+1;
	int sizeZ=ext[5]-ext[4]+1;

    double spc[3];
	bbGetInputIn()->GetSpacing(spc);
    double invSpc[3];
    invSpc[0] = 1/spc[0];
    invSpc[1] = 1/spc[1];
    invSpc[2] = 1/spc[2];

	mimage = vtkImageData::New();
	mimage->SetSpacing(bbGetInputIn()->GetSpacing());
	mimage->SetDimensions(bbGetInputIn()->GetDimensions());
	mimage->SetExtent(bbGetInputIn()->GetExtent());
	mimage->SetOrigin(bbGetInputIn()->GetOrigin());
	//mimage->SetDimensions(sizeX,sizeY,sizeZ);
	//mimage->SetOrigin(0,0,0);
	//mimage->SetExtent( 0 , sizeX-1 , 0 , sizeY-1 , 0, sizeZ-1 );
	//mimage->SetWholeExtent( 0 , sizeX-1 , 0 , sizeY-1 , 0, sizeZ-1 );
	mimage->SetScalarTypeToUnsignedShort();
	//mimage->SetSpacing(spc);
	mimage->AllocateScalars();



	int i,j,k;
    	int sizeLstPointR 	= bbGetInputlstPointR().size();
	int iAxe,sizeAxe	= bbGetInputlstPointX().size();
	double rx,ry,rz;
	double r,rr;
	unsigned short *p;
	int sizeImage 		= sizeX*sizeY*sizeZ;
	double px,py,pz;
	double px1,py1,pz1;
	double px2,py2,pz2;

	// Clean image
	p = (unsigned short*)mimage->GetScalarPointer (0, 0, 0);
	for ( i=0 ; i<sizeImage ; i++)
	{
		*p = 0;
		p++;
	}

	for (iAxe=0 ; iAxe<sizeAxe; iAxe++)
	 {
		if (iAxe<sizeLstPointR)
		{
			r = bbGetInputlstPointR()[ iAxe ]* invSpc[0];
		} else {
			if (bbGetInputlstPointR().size()>=1)
			{
				r = bbGetInputlstPointR()[ bbGetInputlstPointR().size()-1 ]    * invSpc[0];
			} else {
				r = 1;
			}
		}
		px = bbGetInputlstPointX()[iAxe] * invSpc[0];
		py = bbGetInputlstPointY()[iAxe] * invSpc[1];
		pz = bbGetInputlstPointZ()[iAxe] * invSpc[2];
		px1 = px - r;
		py1 = py - r;
		pz1 = pz - r;
		px2 = px + r;
		py2 = py + r;
		pz2 = pz + r;
		rr=r*r;

		for ( i=px1 ; i<=px2 ; i++ )
		{
			rx	=	i - px;
			rx	=	rx*rx;
			for ( j=py1 ; j<py2 ; j++ )
			{
				ry	=	j - py;
				ry	=	ry*ry;
				for ( k=pz1 ; k<pz2 ; k++ )
				{
					if ( (i>=0) && (i<sizeX) && (j>=0) && (j<sizeY) &&(k>=0) && (k<sizeZ) ){
						p = (unsigned short*)mimage->GetScalarPointer (i, j, k);
						if (*p==0)
						{
							rz	=	k - pz;
							rz	=	rz*rz;
							if ( rx + ry + rz <= rr )
							{
								*p=255;
							}
						} // *p==0
					} // inside point
				} //for k
			} //for j
		} //for i
	} // for iAxe
    bbSetOutputOut( mimage );

	printf("EED AxeVolume::Process end \n");

}


	//-----------------------------------------------------------------
	void AxeVolume::bbUserSetDefaultValues()
	{
		mimage=NULL;
	}

	//-----------------------------------------------------------------
	void AxeVolume::bbUserInitializeProcessing()
	{
	}

	//-----------------------------------------------------------------
	void AxeVolume::bbUserFinalizeProcessing()
	{
	}

	//-----------------------------------------------------------------

}
// EO namespace bbcreaMaracasVisu


